"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const general_1 = require("../../helpers/general");
function AsyncRetry(retries, restMs = 100) {
    return general_1.generateFunctionDecorator('AsyncRetry', decorator, retries, restMs);
}
exports.AsyncRetry = AsyncRetry;
function decorator(fn, retries, restMs) {
    return function (...args) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            return retry(this, fn, args, resolve, reject, retries, 1, restMs);
        }));
    };
}
function retry(ctx, fn, args, resolve, reject, maxRetries, attempt, restMs) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = yield fn.apply(ctx, args);
            return resolve(response);
        }
        catch (e) {
            if (maxRetries > attempt) {
                setTimeout(() => retry(ctx, fn, args, resolve, reject, maxRetries, attempt + 1, restMs), restMs);
                return;
            }
            return reject(e);
        }
    });
}
//# sourceMappingURL=async-retry.js.map