"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const general_1 = require("../../helpers/general");
const lru_1 = require("../../helpers/lru");
function LruMemoize(maxRecords, hashFn) {
    return general_1.generateFunctionDecorator('LruMemoize', decorator, maxRecords, hashFn);
}
exports.LruMemoize = LruMemoize;
function decorator(fn, maxRecords, hashFn) {
    const cache = new lru_1.LRU(maxRecords);
    return function (...args) {
        const key = hashFn ? hashFn.apply(this, args) : JSON.stringify(args);
        const value = cache.get(key);
        if (!general_1.isUndefined(value)) {
            return value;
        }
        const response = fn.apply(this, args);
        cache.insert(key, response);
        return response;
    };
}
//# sourceMappingURL=lru-memoize.js.map