"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const general_1 = require("../../helpers/general");
function TimeMemoize(timeoutMs, hashFn) {
    return general_1.generateFunctionDecorator('TimeMemoize', decorator, timeoutMs, hashFn);
}
exports.TimeMemoize = TimeMemoize;
function decorator(fn, timeoutMs, hashFn) {
    const cache = {};
    return function (...args) {
        const now = Date.now();
        const key = hashFn ? hashFn.apply(this, args) : JSON.stringify(args);
        if (!general_1.isUndefined(cache[key]) && cache[key].timeoutTimestamp > now) {
            return cache[key].value;
        }
        cache[key] = {
            value: fn.apply(this, args),
            timeoutTimestamp: now + timeoutMs,
        };
        setTimeout(() => delete cache[key], timeoutMs);
        return cache[key].value;
    };
}
//# sourceMappingURL=time-memoize.js.map