"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const general_1 = require("./general");
class LRU {
    constructor(maxRecords) {
        this.maxRecords = maxRecords;
        this.totalStorage = 0;
        this.cache = {};
    }
    get(key) {
        return !general_1.isUndefined(this.cache[key]) ? this.cache[key].value : undefined;
    }
    insert(key, value) {
        if (this.totalStorage >= this.maxRecords) {
            if (!general_1.isUndefined(this.get(key))) {
                this.remove(key);
            }
            else {
                const { key } = this.tail;
                this.remove(key);
            }
        }
        const node = { key, value, next: this.head };
        this.add(key, node);
    }
    remove(key) {
        const node = this.cache[key];
        if (!node)
            return;
        const { prev, next } = node;
        if (prev) {
            prev.next = next;
        }
        else {
            this.head = next;
        }
        if (node === this.tail) {
            this.tail = prev || this.head;
        }
        if (next) {
            next.prev = prev;
        }
        delete this.cache[key];
        --this.totalStorage;
    }
    toString() {
        const out = [];
        for (let node = this.head; node; out.push(node.value), node = node.next)
            ;
        return out.join(', ');
    }
    add(key, node) {
        if (this.head) {
            this.head.prev = node;
        }
        if (!this.tail) {
            this.tail = node;
        }
        this.cache[key] = this.head = node;
        ++this.totalStorage;
    }
}
exports.LRU = LRU;
//# sourceMappingURL=lru.js.map